
local lfs = require "lfs"
local eqg = require "luaeqg"

local list = iup.list{visiblelines = 10, expand = "VERTICAL", visiblecolumns = 14, sort = "YES"}

local filter = iup.text{visiblecolumns = 13, value = ""}

function filter:valuechanged_cb()
	local path = search_path
	if path then
		UpdateDirList(path)
	end
end

function UpdateDirList(path)
	list[1] = nil
	list.autoredraw = "NO"
	local i = 1
	local f = filter.value
	if f:len() > 0 then
		f = f:gsub("%.", "%%%.")
		if f:find("%%", -1) then
			f = f .. "%"
		end
		for str in lfs.dir(path) do
			if str:find(f) and (str:find("%.eqg", -4) or str:find("%.s3d", -4)) then
				list[i] = str
				i = i + 1
			end
		end
	else
		for str in lfs.dir(path) do
			if str:find("%.eqg", -4) or str:find("%.s3d", -4) then
				list[i] = str
				i = i + 1
			end
		end
	end
	list.autoredraw = "YES"
end

function list:action(str, pos, state)
	if state == 1 then
		local path = search_path .."\\".. str
		UpdateFileList(path)
	end
end

local search_dlg, search_bar, search_filter
local function FirstMatchSearch()
	local n = 1
	local base = search_path .. "\\"
	return function()
		local filename = list[n]
		if not filename then
			iup.SetIdle(nil)
			search_dlg:hide()
			return
		end
		local path = base .. filename
		eqg.CloseDirectory(open_dir)
		open_path = path
		local s, dir = pcall(eqg.LoadDirectory, path)
		if not s then
			error_popup(dir)
			iup.SetIdle(nil)
			search_dlg:hide()
			return
		end
		open_dir = dir
		--dir open, check file names
		for i, ent in ipairs(dir) do
			local name = ent.name
			if name:find(search_filter) then
				file_filter.value = ""
				UpdateFileList(path)
				SetTop(name)
				iup.SetIdle(nil)
				search_dlg:hide()
				return
			end
		end
		--advance
		search_bar.value = n
		n = n + 1
	end
end

local exhaust_list
local function ExhaustiveSearch()
	local n = 1
	local m = 1
	local base = search_path .. "\\"
	return function()
		local filename = list[n]
		if not filename then
			iup.SetIdle(nil)
			search_dlg:hide()
			return
		end
		local path = base .. filename
		eqg.CloseDirectory(open_dir)
		open_path = path
		local s, dir = pcall(eqg.LoadDirectory, path)
		if not s then
			error_popup(dir)
			iup.SetIdle(nil)
			search_dlg:hide()
			return
		end
		open_dir = dir
		--dir open, check file names
		for i, ent in ipairs(dir) do
			local name = ent.name
			if name:find(search_filter) then
				exhaust_list[m] = filename .." - ".. name
				m = m + 1
			end
		end
		--advance
		search_bar.value = n
		n = n + 1
	end
end

local function Search()
	if not search_path then return end
	local text = iup.text{visiblecolumns = 14, expand = "HORIZONTAL"}
	local box = iup.vbox{iup.label{title = "Text to match:"}, text; gap = 2}
	local tog = iup.toggle{title = "Stop at first match", value = "ON"}
	local but = iup.button{title = "Search", padding = "10x0", expand = "HORIZONTAL"}
	local dlg = iup.dialog{iup.vbox{box, tog, but; nmargin = "15x15", gap = 15};
		title = "Multi-Archive Search", size = "200x95"}
	local str, exhaustive
	function but:action()
		if text.value:len() > 0 then
			str = text.value
			exhaustive = (tog.value == "OFF")
		end
		dlg:hide()
	end
	function dlg:k_any(key)
		if key == iup.K_CR then
			but:action()
		end
	end
	iup.Popup(dlg)
	iup.Destroy(dlg)
	if str then
		local num = list.count
		local prog = iup.progressbar{min = 0, max = num, expand = "HORIZONTAL"}
		prog.value = 0
		local but = iup.button{title = "Cancel", action = function()
			iup.SetIdle(nil)
			search_dlg:hide()
		end}
		local dlg = iup.dialog{iup.vbox{iup.hbox{prog; alignment = "ACENTER", expand = "HORIZONTAL"},
			but; alignment = "ACENTER", nmargin = "10x10", gap = 10}; title = "Searching...", size = "200x66"}
		search_bar = prog
		search_filter = str
		if exhaustive then
			exhaust_list = iup.list{size = "178x170"}
			exhaust_dlg = iup.dialog{iup.vbox{exhaust_list; alignment = "ACENTER", nmargin = "10x10", expand = "BOTH"};
				title = "Search Results", size = "200x200"}
			exhaust_dlg:show()
			iup.SetIdle(ExhaustiveSearch())
		else
			iup.SetIdle(FirstMatchSearch())
		end
		search_dlg = dlg
		iup.Popup(dlg)
		iup.Destroy(dlg)
	end
end

function list:button_cb(button, pressed, x, y)
	if button == iup.BUTTON3 and pressed == 0 then
		local mx, my = iup.GetGlobal("CURSORPOS"):match("(%d+)x(%d+)")
		local menu = iup.menu{
			iup.item{title = "Search All...", action = Search},
		}
		iup.Popup(menu, mx, my)
		iup.Destroy(menu)
	end
end

return iup.vbox{iup.hbox{iup.label{title = "Filter"}, filter; alignment = "ACENTER", gap = 5}, list;
	alignment = "ACENTER", gap = 5}
