
local eqg = require "luaeqg"
local lfs = require "lfs"

function assert(result, msg)
	if result then return result end
	local err = iup.messagedlg{buttons = "OK", dialogtype = "ERROR", title = "Error", value = msg}
	iup.Popup(err)
	iup.Close()
end

function error_popup(msg)
	local err = iup.messagedlg{buttons = "OK", dialogtype = "ERROR", title = "Error", value = msg}
	iup.Popup(err)
	iup.Destroy(err)
end

local dir_list = require "dir_list"
local file_list = require "file_list"
local canvas = require "canvas"

local function SetSearchFolder()
	local dlg = iup.filedlg{title = "Select EQG Search Folder", dialogtype = "DIR",
		directory = search_path or lfs.currentdir()}
	iup.Popup(dlg)
	if dlg.status == "0" then
		local path = dlg.value
		if path then
			search_path = path
			UpdateDirList(path)
			local f = assert(io.open("settings.lua", "w+"))
			f:write("\nreturn \"".. (path:gsub("\\", "\\\\")) .."\"\n")
			f:close()
		end
	end
end

local function LoadSettings()
	local settings = loadfile("settings.lua")
	if settings then
		local path = settings()
		if path then
			search_path = path
			UpdateDirList(path)
			return
		end
	end
	SetSearchFolder()
end

LoadSettings()
LoadSettings = nil

local menu = iup.menu{
	iup.submenu{
		title = "&File";
		iup.menu{
			iup.item{title = "Set Search Folder", action = SetSearchFolder},
			iup.separator{},
			iup.item{title = "Quit", action = function() return iup.CLOSE end},
		},
	},
}

local window = assert(iup.dialog{iup.hbox{dir_list, file_list, canvas; nmargin = "10x10", gap = 10};
	title = "EQGExport v1.1", menu = menu, size = "525x250"})

function window:k_any(key)
	if key == iup.K_ESC then
		return iup.CLOSE
	end
end

window:show()
iup.MainLoop()

eqg.CloseDirectory(open_dir)
