
#include "lua.hpp"
#include <cstdio>
#include <windows.h>

#include "iup.h"
#include "iuplua.h"
#include "iupcontrols.h"
#include "iupluacontrols.h"
#include "iupgl.h"
#include "iupluagl.h"

#include <gl/gl.h>
#include "FreeImagePlus.h"

fipImage cur_image;

int SetCurImage(lua_State* L)
{
	if (!lua_isuserdata(L, 1))
		return luaL_argerror(L, 1, "expected data ptr");
	int len = luaL_checkinteger(L, 2); //inflated_len

	byte* ptr = (byte*)lua_touserdata(L, 1);

	fipMemoryIO mem(ptr, len);

	cur_image.clear();
	lua_pushboolean(L, (cur_image.loadFromMemory(mem) && cur_image.convertTo32Bits()));

	return 1;
}

int DrawCurImage(lua_State* L)
{
	float zoom = static_cast<float>(luaL_checknumber(L, 1));

	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode(GL_TEXTURE);
	glViewport(0, 0, 200, 200);
	glLoadIdentity();

	glPixelZoom(zoom, zoom);

	if (cur_image.isValid())
	{
		glDrawPixels(cur_image.getWidth(), cur_image.getHeight(),
			0x80E1, GL_UNSIGNED_BYTE, cur_image.accessPixels());
		//windows' gl.h doesn't define GL_BGRA for whatever reason
	}

	return 0;
}

int CALLBACK WinMain(
  _In_  HINSTANCE hInstance,
  _In_  HINSTANCE hPrevInstance,
  _In_  LPSTR lpCmdLine,
  _In_  int nCmdShow
)
{
	lua_State* L = luaL_newstate();
	luaL_openlibs(L);

	lua_pushcfunction(L, SetCurImage);
	lua_setglobal(L, "SetCurImage");
	lua_pushcfunction(L, DrawCurImage);
	lua_setglobal(L, "DrawCurImage");

	IupOpen(nullptr, nullptr);
	IupControlsOpen();
	IupGLCanvasOpen();
	iuplua_open(L);
	iupcontrolslua_open(L);
	iupgllua_open(L);

	if (luaL_loadfile(L, "main.lua") != 0)
	{
		printf("Error: Could not load script: %s\n", lua_tostring(L, -1));
		lua_close(L);
		return 1;
	}

	if (lua_pcall(L, 0, 0, 0) != 0)
		printf("Runtime error: %s\n", lua_tostring(L, -1));
	lua_close(L);

    return 0;
}
